#include <iostream>
#include "opencl.hpp"

static struct {
   cl_platform_info info;
   const char* name;
} cl_platform_params[] = {
   {CL_PLATFORM_PROFILE, "CL_PLATFORM_PROFILE"},
   {CL_PLATFORM_VERSION, "CL_PLATFORM_VERSION"},
   {CL_PLATFORM_NAME, "CL_PLATFORM_NAME"},
   {CL_PLATFORM_VENDOR, "CL_PLATFORM_VENDOR"},
   {CL_PLATFORM_EXTENSIONS, "CL_PLATFORM_EXTENSIONS"},
};
static const unsigned int cl_platform_parameters = sizeof(cl_platform_params) /
   sizeof(cl_platform_params[0]);

template<typename T>
void get_device_info(cl_device_id device, cl_device_info param_name,
      const char param_string[]) {
   T param_value;
   CHECK_OPENCL(clGetDeviceInfo, device, param_name, sizeof param_value,
      &param_value, 0);
   std::cout << "      " << param_string << ": " << param_value << std::endl;
}
template<>
void get_device_info<size_t[3]>(cl_device_id device, cl_device_info param_name,
      const char param_string[]) {
   size_t dimensions[3];
   CHECK_OPENCL(clGetDeviceInfo, device, param_name, sizeof dimensions,
      dimensions, 0);
   std::cout << "      " << param_string << ": " <<
      dimensions[0] << " x " << dimensions[1] << " x " << dimensions[2] <<
      std::endl;
}
#define GET_DEVICE_INFO(device, param_name, param_type) \
   get_device_info<param_type>((device), (param_name), #param_name)

int main() {
   cl_uint number_of_platforms;
   CHECK_OPENCL(clGetPlatformIDs, 0, 0, &number_of_platforms);
   cl_platform_id platforms[number_of_platforms];
   CHECK_OPENCL(clGetPlatformIDs, number_of_platforms, platforms, 0);
   if (number_of_platforms == 1) {
      std::cout << "Just one platform found." << std::endl;
   } else {
      std::cout << number_of_platforms << " platforms found." << std::endl;
   }

   for (cl_uint index = 0; index < number_of_platforms; ++index) {
      std::cout << "Platform " << index << ":" << std::endl;
      for (unsigned int p = 0; p < cl_platform_parameters; ++p) {
	 cl_platform_info info = cl_platform_params[p].info;
	 const char* name = cl_platform_params[p].name;
	 char buf[512];
	 CHECK_OPENCL(clGetPlatformInfo, platforms[index],
	    info, sizeof buf, buf, 0);
	 std::cout << "   " << name << ": " << buf << std::endl;
      }
      cl_uint number_of_devices;
      CHECK_OPENCL(clGetDeviceIDs, platforms[index],
	 CL_DEVICE_TYPE_ALL, 0, 0, &number_of_devices);
      cl_device_id device_ids[number_of_devices];
      CHECK_OPENCL(clGetDeviceIDs, platforms[index],
	 CL_DEVICE_TYPE_ALL, number_of_devices, device_ids, 0);
      if (number_of_devices == 1) {
	 std::cout << "   One device found." << std::endl;
      } else {
	 std::cout << "   " << number_of_devices <<
	    " devices found." << std::endl;
      }
      for (cl_uint device = 0; device < number_of_devices; ++device) {
	 std::cout << "   Device " << device << ":" << std::endl;
	 cl_device_id id = device_ids[device];
	 GET_DEVICE_INFO(id, CL_DEVICE_ADDRESS_BITS, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_AVAILABLE, cl_bool);
	 GET_DEVICE_INFO(id, CL_DEVICE_COMPILER_AVAILABLE, cl_bool);
	 GET_DEVICE_INFO(id, CL_DEVICE_ENDIAN_LITTLE, cl_bool);
	 GET_DEVICE_INFO(id, CL_DEVICE_ERROR_CORRECTION_SUPPORT, cl_bool);
	 GET_DEVICE_INFO(id, CL_DEVICE_EXECUTION_CAPABILITIES,
	    cl_device_exec_capabilities);
	 GET_DEVICE_INFO(id, CL_DEVICE_EXTENSIONS, char[512]);
	 GET_DEVICE_INFO(id, CL_DEVICE_GLOBAL_MEM_CACHE_SIZE, cl_ulong);
	 GET_DEVICE_INFO(id, CL_DEVICE_GLOBAL_MEM_CACHE_TYPE,
	    cl_device_mem_cache_type);
	 GET_DEVICE_INFO(id, CL_DEVICE_GLOBAL_MEM_CACHELINE_SIZE, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_GLOBAL_MEM_SIZE, cl_ulong);
	 GET_DEVICE_INFO(id, CL_DEVICE_HOST_UNIFIED_MEMORY, cl_bool);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_COMPUTE_UNITS, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_CONSTANT_ARGS, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE, cl_ulong);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_MEM_ALLOC_SIZE, cl_ulong);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_WORK_GROUP_SIZE, size_t);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_MAX_WORK_ITEM_SIZES, size_t[3]);
	 GET_DEVICE_INFO(id, CL_DEVICE_NAME, char[512]);
	 GET_DEVICE_INFO(id, CL_DEVICE_OPENCL_C_VERSION, char[512]);
	 GET_DEVICE_INFO(id, CL_DEVICE_VENDOR, char[512]);
	 GET_DEVICE_INFO(id, CL_DEVICE_VENDOR_ID, cl_uint);
	 GET_DEVICE_INFO(id, CL_DEVICE_VERSION, char[512]);
	 GET_DEVICE_INFO(id, CL_DRIVER_VERSION, char[512]);
      }
   }
}
